@@ -36,10 +36,10 @@ Features
 
 
 .. _column_property:
-    http://docs.sqlalchemy.org/en/latest/orm/mapper_config.html#using-column-property
+    https://docs.sqlalchemy.org/en/latest/orm/mapper_config.html#using-column-property
 .. _counter_culture: https://github.com/magnusvk/counter_culture
 .. _stackoverflow reply by Michael Bayer:
-    http://stackoverflow.com/questions/13693872/
+    https://stackoverflow.com/a/13765857/520932
 
 
 Simple aggregates
@@ -72,9 +72,9 @@ Simple aggregates
         thread_id = sa.Column(sa.Integer, sa.ForeignKey(Thread.id))
 
 
-    thread = Thread(name=u'SQLAlchemy development')
-    thread.comments.append(Comment(u'Going good!'))
-    thread.comments.append(Comment(u'Great new features!'))
+    thread = Thread(name='SQLAlchemy development')
+    thread.comments.append(Comment('Going good!'))
+    thread.comments.append(Comment('Great new features!'))
 
     session.add(thread)
     session.commit()
@@ -138,7 +138,7 @@ Now the net_worth column of Catalog model will be automatically whenever:
 
 
     catalog = Catalog(
-        name=u'My first catalog',
+        name='My first catalog',
         products=[
             product1,
             product2
@@ -252,8 +252,8 @@ scenarios includes things such as:
 
 
 
-        user = User(name=u'John Matrix')
-        user.groups = [Group(name=u'Group A'), Group(name=u'Group B')]
+        user = User(name='John Matrix')
+        user.groups = [Group(name='Group A'), Group(name='Group B')]
 
         session.add(user)
         session.commit()
@@ -265,7 +265,7 @@ scenarios includes things such as:
 Multi-level aggregates
 ----------------------
 
-Aggregates can span accross multiple relationships. In the following example
+Aggregates can span across multiple relationships. In the following example
 each Catalog has a net_worth which is the sum of all products in all
 categories.
 
@@ -285,7 +285,7 @@ categories.
         def net_worth(self):
             return sa.func.sum(Product.price)
 
-        categories = sa.orm.relationship('Product')
+        categories = sa.orm.relationship('Category')
 
 
     class Category(Base):
@@ -357,7 +357,7 @@ TODO
 
 
 .. _deadlocks:
-    http://mina.naguib.ca/blog/2010/11/22/postgresql-foreign-key-deadlocks.html
+    https://mina.naguib.ca/blog/2010/11/22/postgresql-foreign-key-deadlocks.html
 
 """
 
@@ -365,11 +365,13 @@ TODO
 from collections import defaultdict
 from weakref import WeakKeyDictionary
 
-import six
 import sqlalchemy as sa
+import sqlalchemy.event
+import sqlalchemy.orm
 from sqlalchemy.ext.declarative import declared_attr
 from sqlalchemy.sql.functions import _FunctionGenerator
 
+from .compat import _select_args, get_scalar_subquery
 from .functions.orm import get_column_key
 from .relationships import (
     chained_join,
@@ -377,7 +379,7 @@ from .relationships import (
     select_correlated_expression
 )
 
-aggregated_attrs = WeakKeyDictionary(defaultdict(list))
+aggregated_attrs = WeakKeyDictionary()
 
 
 class AggregatedAttribute(declared_attr):
@@ -389,7 +391,7 @@ class AggregatedAttribute(declared_attr):
         *args,
         **kwargs
     ):
-        super(AggregatedAttribute, self).__init__(fget, *args, **kwargs)
+        super().__init__(fget, *args, **kwargs)
         self.__doc__ = fget.__doc__
         self.column = column
         self.relationship = relationship
@@ -434,7 +436,7 @@ def aggregate_expression(expr, class_):
         return expr(class_)
 
 
-class AggregatedValue(object):
+class AggregatedValue:
     def __init__(self, class_, attr, path, expr):
         self.class_ = class_
         self.attr = attr
@@ -453,7 +455,7 @@ class AggregatedValue(object):
             self.relationships[0].mapper.class_
         )
 
-        return query.as_scalar()
+        return get_scalar_subquery(query)
 
     def update_query(self, objects):
         table = self.class_.__table__
@@ -488,10 +490,9 @@ class AggregatedValue(object):
                 return query.where(
                     local.in_(
                         sa.select(
-                            [remote],
-                            from_obj=[
-                                chained_join(*reversed(self.relationships))
-                            ]
+                            *_select_args(remote)
+                        ).select_from(
+                            chained_join(*reversed(self.relationships))
                         ).where(
                             condition
                         )
@@ -499,7 +500,7 @@ class AggregatedValue(object):
                 )
 
 
-class AggregationManager(object):
+class AggregationManager:
     def __init__(self):
         self.reset()
 
@@ -519,7 +520,7 @@ class AggregationManager(object):
         )
 
     def update_generator_registry(self):
-        for class_, attrs in six.iteritems(aggregated_attrs):
+        for class_, attrs in aggregated_attrs.items():
             for expr, path, column in attrs:
                 value = AggregatedValue(
                     class_=class_,
@@ -535,11 +536,11 @@ class AggregationManager(object):
     def construct_aggregate_queries(self, session, ctx):
         object_dict = defaultdict(list)
         for obj in session:
-            class_ = obj.__class__
-            if class_ in self.generator_registry:
-                object_dict[class_].append(obj)
+            for class_ in self.generator_registry:
+                if isinstance(obj, class_):
+                    object_dict[class_].append(obj)
 
-        for class_, objects in six.iteritems(object_dict):
+        for class_, objects in object_dict.items():
             for aggregate_value in self.generator_registry[class_]:
                 query = aggregate_value.update_query(objects)
                 if query is not None:
