@@ -1,27 +1,27 @@
 from collections import defaultdict
 from itertools import groupby
 
-import six
 import sqlalchemy as sa
 from sqlalchemy.exc import NoInspectionAvailable
 from sqlalchemy.orm import object_session
-from sqlalchemy.schema import MetaData, Table, ForeignKeyConstraint
+from sqlalchemy.schema import ForeignKeyConstraint, MetaData, Table
 
-from .orm import get_column_key, get_mapper, get_tables
 from ..query_chain import QueryChain
+from .database import has_index
+from .orm import _get_class_registry, get_column_key, get_mapper, get_tables
 
 
 def get_foreign_key_values(fk, obj):
-    return dict(
-        (
-            fk.constraint.columns[index].key,
-            getattr(obj, element.column.key)
+    mapper = get_mapper(obj)
+    return {
+        fk.constraint.columns.values()[index]:
+        getattr(obj, element.column.key)
+        if hasattr(obj, element.column.key)
+        else getattr(
+            obj, mapper.get_property_by_column(element.column).key
         )
-        for
-        index, element
-        in
-        enumerate(fk.constraint.elements)
-    )
+        for index, element in enumerate(fk.constraint.elements)
+    }
 
 
 def group_foreign_keys(foreign_keys):
@@ -165,27 +165,25 @@ def merge_references(from_, to, foreign_keys=None):
         old_values = get_foreign_key_values(fk, from_)
         new_values = get_foreign_key_values(fk, to)
         criteria = (
-            getattr(fk.constraint.table.c, key) == value
-            for key, value in six.iteritems(old_values)
+            getattr(fk.constraint.table.c, key.key) == value
+            for key, value in old_values.items()
         )
         try:
             mapper = get_mapper(fk.constraint.table)
         except ValueError:
             query = (
-                fk.constraint.table
-                .update()
+                fk.constraint.table.update()
                 .where(sa.and_(*criteria))
-                .values(new_values)
+                .values(
+                    {key.key: value for key, value in new_values.items()}
+                )
             )
             session.execute(query)
         else:
             (
                 session.query(mapper.class_)
-                .filter_by(**old_values)
-                .update(
-                    new_values,
-                    'evaluate'
-                )
+                .filter(*[k == old_values[k] for k in old_values])
+                .update(new_values, synchronize_session='fetch')
             )
 
 
@@ -265,7 +263,7 @@ def dependent_objects(obj, foreign_keys=None):
     session = object_session(obj)
 
     chain = QueryChain([])
-    classes = obj.__class__._decl_class_registry
+    classes = _get_class_registry(obj.__class__)
 
     for table, keys in group_foreign_keys(foreign_keys):
         keys = list(keys)
@@ -336,7 +334,6 @@ def non_indexed_foreign_keys(metadata, engine=None):
         table = Table(
             table_name,
             reflected_metadata,
-            autoload=True,
             autoload_with=metadata.bind or engine
         )
 
@@ -344,22 +341,13 @@ def non_indexed_foreign_keys(metadata, engine=None):
             if not isinstance(constraint, ForeignKeyConstraint):
                 continue
 
-            if not is_indexed_foreign_key(constraint):
+            if not has_index(constraint):
                 constraints[table.name].append(constraint)
 
     return dict(constraints)
 
 
-def is_indexed_foreign_key(constraint):
-    """
-    Whether or not given foreign key constraint's columns have been indexed.
-
-    :param constraint: ForeignKeyConstraint object to check the indexes
-    """
-    return any(
-        set(column.name for column in index.columns)
-        ==
-        set(constraint.columns)
-        for index
-        in constraint.table.indexes
-    )
+def get_fk_constraint_for_columns(table, *columns):
+    for constraint in table.constraints:
+        if list(constraint.columns.values()) == list(columns):
+            return constraint
