@@ -1,11 +1,10 @@
-# -*- coding: utf-8 -*-
 import base64
 import datetime
 import json
 import os
+import warnings
 
-import six
-from sqlalchemy.types import String, TypeDecorator
+from sqlalchemy.types import LargeBinary, String, TypeDecorator
 
 from sqlalchemy_utils.exceptions import ImproperlyConfigured
 from sqlalchemy_utils.types.encrypted.padding import PADDING_MECHANISM
@@ -15,17 +14,18 @@ from sqlalchemy_utils.types.scalar_coercible import ScalarCoercible
 cryptography = None
 try:
     import cryptography
+    from cryptography.exceptions import InvalidTag
+    from cryptography.fernet import Fernet
     from cryptography.hazmat.backends import default_backend
     from cryptography.hazmat.primitives import hashes
     from cryptography.hazmat.primitives.ciphers import (
-        Cipher, algorithms, modes
+        algorithms,
+        Cipher,
+        modes
     )
-    from cryptography.fernet import Fernet
-    from cryptography.exceptions import InvalidTag
 except ImportError:
     pass
 
-
 dateutil = None
 try:
     import dateutil
@@ -38,7 +38,7 @@ class InvalidCiphertextError(Exception):
     pass
 
 
-class EncryptionDecryptionBaseEngine(object):
+class EncryptionDecryptionBaseEngine:
     """A base encryption and decryption engine.
 
     This class must be sub-classed in order to create
@@ -46,7 +46,7 @@ class EncryptionDecryptionBaseEngine(object):
     """
 
     def _update_key(self, key):
-        if isinstance(key, six.string_types):
+        if isinstance(key, str):
             key = key.encode()
         digest = hashes.Hash(hashes.SHA256(), backend=default_backend())
         digest.update(key)
@@ -88,7 +88,7 @@ class AesEngine(EncryptionDecryptionBaseEngine):
 
     def _set_padding_mechanism(self, padding_mechanism=None):
         """Set the padding mechanism."""
-        if isinstance(padding_mechanism, six.string_types):
+        if isinstance(padding_mechanism, str):
             if padding_mechanism not in PADDING_MECHANISM.keys():
                 raise ImproperlyConfigured(
                     "There is not padding mechanism with name {}".format(
@@ -103,9 +103,9 @@ class AesEngine(EncryptionDecryptionBaseEngine):
         self.padding_engine = padding_class(self.BLOCK_SIZE)
 
     def encrypt(self, value):
-        if not isinstance(value, six.string_types):
+        if not isinstance(value, str):
             value = repr(value)
-        if isinstance(value, six.text_type):
+        if isinstance(value, str):
             value = str(value)
         value = value.encode()
         value = self.padding_engine.pad(value)
@@ -115,13 +115,13 @@ class AesEngine(EncryptionDecryptionBaseEngine):
         return encrypted.decode('utf-8')
 
     def decrypt(self, value):
-        if isinstance(value, six.text_type):
+        if isinstance(value, str):
             value = str(value)
         decryptor = self.cipher.decryptor()
         decrypted = base64.b64decode(value)
         decrypted = decryptor.update(decrypted) + decryptor.finalize()
         decrypted = self.padding_engine.unpad(decrypted)
-        if not isinstance(decrypted, six.string_types):
+        if not isinstance(decrypted, str):
             try:
                 decrypted = decrypted.decode('utf-8')
             except UnicodeDecodeError:
@@ -151,9 +151,9 @@ class AesGcmEngine(EncryptionDecryptionBaseEngine):
         self.secret_key = parent_class_key
 
     def encrypt(self, value):
-        if not isinstance(value, six.string_types):
+        if not isinstance(value, str):
             value = repr(value)
-        if isinstance(value, six.text_type):
+        if isinstance(value, str):
             value = str(value)
         value = value.encode()
         iv = os.urandom(self.IV_BYTES_NEEDED)
@@ -169,7 +169,7 @@ class AesGcmEngine(EncryptionDecryptionBaseEngine):
         return encrypted.decode('utf-8')
 
     def decrypt(self, value):
-        if isinstance(value, six.text_type):
+        if isinstance(value, str):
             value = str(value)
         decrypted = base64.b64decode(value)
         if len(decrypted) < self.IV_BYTES_NEEDED + self.TAG_SIZE_BYTES:
@@ -188,7 +188,7 @@ class AesGcmEngine(EncryptionDecryptionBaseEngine):
             decrypted = decryptor.update(decrypted) + decryptor.finalize()
         except InvalidTag:
             raise InvalidCiphertextError()
-        if not isinstance(decrypted, six.string_types):
+        if not isinstance(decrypted, str):
             try:
                 decrypted = decrypted.decode('utf-8')
             except UnicodeDecodeError:
@@ -204,24 +204,24 @@ class FernetEngine(EncryptionDecryptionBaseEngine):
         self.fernet = Fernet(self.secret_key)
 
     def encrypt(self, value):
-        if not isinstance(value, six.string_types):
+        if not isinstance(value, str):
             value = repr(value)
-        if isinstance(value, six.text_type):
+        if isinstance(value, str):
             value = str(value)
         value = value.encode()
         encrypted = self.fernet.encrypt(value)
         return encrypted.decode('utf-8')
 
     def decrypt(self, value):
-        if isinstance(value, six.text_type):
+        if isinstance(value, str):
             value = str(value)
         decrypted = self.fernet.decrypt(value.encode())
-        if not isinstance(decrypted, six.string_types):
+        if not isinstance(decrypted, str):
             decrypted = decrypted.decode('utf-8')
         return decrypted
 
 
-class EncryptedType(TypeDecorator, ScalarCoercible):
+class StringEncryptedType(TypeDecorator, ScalarCoercible):
     """
     EncryptedType provides a way to encrypt and decrypt values,
     to and from databases, that their type is a basic SQLAlchemy type.
@@ -257,7 +257,11 @@ class EncryptedType(TypeDecorator, ScalarCoercible):
 
         import sqlalchemy as sa
         from sqlalchemy import create_engine
-        from sqlalchemy.ext.declarative import declarative_base
+        try:
+            from sqlalchemy.orm import declarative_base
+        except ImportError:
+            # sqlalchemy 1.3
+            from sqlalchemy.ext.declarative import declarative_base
         from sqlalchemy.orm import sessionmaker
 
         from sqlalchemy_utils import EncryptedType
@@ -301,7 +305,7 @@ class EncryptedType(TypeDecorator, ScalarCoercible):
         session = Session()
 
         # example
-        user_name = u'secret_user'
+        user_name = 'secret_user'
         test_token = 'atesttoken'
         active = True
         num_of_accounts = 2
@@ -345,17 +349,23 @@ class EncryptedType(TypeDecorator, ScalarCoercible):
                 sa.Unicode, get_key))
 
     """
-
     impl = String
+    cache_ok = True
 
-    def __init__(self, type_in=None, key=None,
-                 engine=None, padding=None, **kwargs):
+    def __init__(
+        self,
+        type_in=None,
+        key=None,
+        engine=None,
+        padding=None,
+        **kwargs
+    ):
         """Initialization."""
         if not cryptography:
             raise ImproperlyConfigured(
                 "'cryptography' is required to use EncryptedType"
             )
-        super(EncryptedType, self).__init__(**kwargs)
+        super().__init__(**kwargs)
         # set the underlying type
         if type_in is None:
             type_in = String()
@@ -403,7 +413,7 @@ class EncryptedType(TypeDecorator, ScalarCoercible):
                     value = value.isoformat()
 
                 elif issubclass(type_, JSONType):
-                    value = six.text_type(json.dumps(value))
+                    value = json.dumps(value)
 
             return self.engine.encrypt(value)
 
@@ -446,7 +456,31 @@ class EncryptedType(TypeDecorator, ScalarCoercible):
         return value
 
 
-class DatetimeHandler(object):
+class EncryptedType(StringEncryptedType):
+    impl = LargeBinary
+
+    def __init__(self, *args, **kwargs):
+        warnings.warn(
+            "The 'EncryptedType' class will change implementation from "
+            "'LargeBinary' to 'String' in a future version. Use "
+            "'StringEncryptedType' to use the 'String' implementation.",
+            DeprecationWarning)
+        super().__init__(*args, **kwargs)
+
+    def process_bind_param(self, value, dialect):
+        value = super().process_bind_param(value=value, dialect=dialect)
+        if isinstance(value, str):
+            value = value.encode()
+        return value
+
+    def process_result_value(self, value, dialect):
+        if isinstance(value, bytes):
+            value = value.decode()
+            value = super().process_result_value(value=value, dialect=dialect)
+        return value
+
+
+class DatetimeHandler:
     """
     DatetimeHandler is responsible for parsing strings and
     returning the appropriate date, datetime or time objects.
