@@ -2,7 +2,9 @@ from datetime import datetime
 
 import pytest
 import sqlalchemy as sa
+from dateutil import tz
 
+from sqlalchemy_utils.compat import _select_args
 from sqlalchemy_utils.types import arrow
 
 
@@ -12,6 +14,8 @@ def Article(Base):
         __tablename__ = 'article'
         id = sa.Column(sa.Integer, primary_key=True)
         created_at = sa.Column(arrow.ArrowType)
+        published_at = sa.Column(arrow.ArrowType(timezone=True))
+        published_at_dt = sa.Column(sa.DateTime(timezone=True))
     return Article
 
 
@@ -21,8 +25,7 @@ def init_models(Article):
 
 
 @pytest.mark.skipif('arrow.arrow is None')
-class TestArrowDateTimeType(object):
-
+class TestArrowDateTimeType:
     def test_parameter_processing(self, session, Article):
         article = Article(
             created_at=arrow.arrow.get(datetime(2000, 11, 1))
@@ -36,7 +39,7 @@ class TestArrowDateTimeType(object):
 
     def test_string_coercion(self, Article):
         article = Article(
-            created_at='1367900664'
+            created_at='2013-01-01'
         )
         assert article.created_at.year == 2013
 
@@ -60,4 +63,23 @@ class TestArrowDateTimeType(object):
     def test_literal_param(self, session, Article):
         clause = Article.created_at > '2015-01-01'
         compiled = str(clause.compile(compile_kwargs={"literal_binds": True}))
-        assert compiled == 'article.created_at > 2015-01-01'
+        assert compiled == "article.created_at > '2015-01-01'"
+
+    @pytest.mark.usefixtures('postgresql_dsn')
+    def test_timezone(self, session, Article):
+        timezone = tz.gettz('Europe/Stockholm')
+        dt = arrow.arrow.get(datetime(2015, 1, 1, 15, 30, 45), timezone)
+        article = Article(published_at=dt, published_at_dt=dt.datetime)
+
+        session.add(article)
+        session.commit()
+        session.expunge_all()
+
+        item = session.query(Article).one()
+        assert item.published_at.datetime == item.published_at_dt
+        assert item.published_at.to(timezone) == dt
+
+    def test_compilation(self, Article, session):
+        query = sa.select(*_select_args(Article.created_at))
+        # the type should be cacheable and not throw exception
+        session.execute(query)
